## Brian Blaylock
## May 13, 2021

import matplotlib.pyplot as plt
import matplotlib.patheffects as path_effects

def HerbieColors():
    return dict(body='#f0ead2', red='#88211b', blue='#0c3576', white='#ffffff', black='#000000')

def HerbieLogo(white_line=False):
    """Logo of Herbie The Love Bug"""
    colors = HerbieColors()

    plt.figure(figsize=[5,5], facecolor=colors['body'])

    plt.axis([-10, 10, -10, 10])

    if white_line:
        plt.axvline(4, lw=40, color=colors['white'])
    plt.axvline(2.5, lw=20, color=colors['red'])
    plt.axvline(5.5, lw=40, color=colors['blue'])
    

    c=plt.Circle((0, 0), radius=6, ec='k', fc='w', zorder=10, linewidth=3)
    plt.gca().add_artist(c)

    plt.text(0,0, '53', fontsize=110, fontweight='bold', va='center_baseline', ha='center', zorder=11)

    plt.gca().set_facecolor(colors['body'])
    plt.gca().margins(0)

    plt.gca().get_xaxis().set_visible(False)
    plt.gca().get_yaxis().set_visible(False)

    plt.gca().spines['bottom'].set_visible(False)
    plt.gca().spines['top'].set_visible(False)
    plt.gca().spines['left'].set_visible(False)
    plt.gca().spines['right'].set_visible(False)

    return plt.gca()

def HerbieLogo2(white_line=False, text_color='body', text_stroke='black'):
    """Logo of Herbie The Love Bug"""
    colors = HerbieColors()

    plt.figure(figsize=[5,3], facecolor=colors['body'])

    plt.axis([1.5, 20, -10, 10])

    if white_line:
        plt.axvline(4, lw=40, color=colors['white'])
    plt.axvline(2.5, lw=20, color=colors['red'])
    plt.axvline(5.5, lw=40, color=colors['blue'])

    
    if text_color in colors:
        text_color = colors[text_color]
    if text_stroke in colors:
        text_stroke = colors[text_stroke]

    text = plt.text(8, 0, 'Herbie',
                    fontsize=110, fontweight='bold', 
                    color=text_color,
                    va='center_baseline', ha='left',
                    zorder=11)
    
    if text_stroke is not None:
        text.set_path_effects([path_effects.Stroke(linewidth=3,
                                                foreground=text_stroke),
                            path_effects.Normal()])

    plt.gca().set_facecolor(colors['body'])
    plt.gca().margins(0)

    plt.gca().get_xaxis().set_visible(False)
    plt.gca().get_yaxis().set_visible(False)

    plt.gca().spines['bottom'].set_visible(False)
    plt.gca().spines['top'].set_visible(False)
    plt.gca().spines['left'].set_visible(False)
    plt.gca().spines['right'].set_visible(False)

    return plt.gca()


## Brian Blaylock
## April 28, 2021

import warnings
import configparser
import os
from pathlib import Path


########################################################################
# Load custom xarray accessors
try:
    import herbie.accessors
except:
    warnings.warn(
        "herbie xarray accessors could not be imported."
        "You are probably missing the Carpenter_Workshop."
        "If you want to use these functions, try"
        "`pip install git+https://github.com/blaylockbk/Carpenter_Workshop.git`"
        )
    pass

########################################################################
# Append Path object with my custom expand method so user can use
# environment variables in the config file (e.g., ${HOME}).
def _expand(self):
    """
    Fully expand and resolve the Path with the given environment variables.
    
    Example
    -------
    >>> Path('$HOME').expand()
    >>> PosixPath('/p/home/blaylock')
    """
    return Path(os.path.expandvars(self)).expanduser().resolve()

Path.expand = _expand

########################################################################
# Configure Herbie
# Configuration file is save in `~/config/herbie/config.cfg`
# `_default_save_dir` is the default path to save GRIB2 files.
config = configparser.ConfigParser()
_config_path = Path('~/.config/herbie/config.cfg').expand()

########################################################################
# Default Configuration Values
defaults = dict(
    data_dir = str(Path('~/data').expand()),
    priority = ','.join(['aws', 'nomads', 'google', 'azure', 'pando', 'pando2'])
)

########################################################################
# If a config file isn't found, make one
if not _config_path.exists():
    _config_path.parent.mkdir(parents=True, exist_ok=True)
    _config_path.touch()
    config.read(_config_path)
    config.add_section('download')
    config.set('download', 'default_save_dir', defaults['data_dir'])
    config.set('download', 'default_priority', defaults['priority'])
    with open(_config_path, 'w') as configfile:
        config.write(configfile)
    print(f'⚙ Created config file [{_config_path}]',
          f'with default download directory set as [{defaults["data_dir"]}]', 
          f'and default source priority as [{defaults["priority"]}]')


########################################################################
# Read the config file
config.read(_config_path)

try:
    _default_save_dir = Path(config.get('download', 'default_save_dir')).expand()
    _default_priority = config.get('download', 'default_priority').split(',')
except:
    print(f'🦁🐯🐻 oh my! {_config_path} looks weird,',
          f'but I will add new settings')
    try:
        config.add_section('download')
    except:
        pass  # section already exists
    config.set('download', 'default_save_dir', defaults['data_dir'])
    config.set('download', 'default_priority', defaults['priority'])
    with open(_config_path, 'w') as configfile:
        config.write(configfile)
    _default_save_dir = Path(config.get('download', 'default_save_dir'))
    _default_priority = config.get('download', 'default_priority').split(',')